import java.util.*;

public class Knoten {
  private Datenliste datenliste;
  private int anzahlElemente; 
  private Knoten linkesKind;
  private Knoten rechtesKind;
  private int splitMerkmal;
  private double splitSchwellenwert;
  private String label;    // haeufigste Klasse in der zugeordneten Datenliste
  private String wert;     // Wert bzgl. des Tests im Vaterknoten; fr Ausgabe bentigt
  
  // Konstruktur zur Erzeugung eines Knotens
  public Knoten(Datenliste d, String split) {
    datenliste = d;
    anzahlElemente = datenliste.getAnzahlElemente();
    linkesKind = null;
    rechtesKind = null;
    splitMerkmal = 0;
    splitSchwellenwert = 0;
    label = datenliste.haeufigsteKlasse();
    wert = split;
  }
  
  // Bestimmung des besten Merkmals (niedrigste FKR) fr den nchsten Split
  private void bestimmeSplitMerkmal() {
    double min_FKR = 1.0;
    // iteriere ber alle Merkmale   
    double aktuelle_FKR = 1.0;
    for (int m=0; m<datenliste.getAnzahlMerkmale(); m++) {
      // iteriere ber alle mglichen Schwellenwerte (= Merkmalsausprgungen) 
      for (int s=0; s<datenliste.getAnzahlElemente(); s++ ) {
        double schwellenwert = datenliste.getElement(s).getAuspraegung(m);
        aktuelle_FKR = datenliste.berechneFKR(m,schwellenwert);
        if (aktuelle_FKR < min_FKR) {
          min_FKR = aktuelle_FKR;
          splitMerkmal = m;
          splitSchwellenwert = schwellenwert;
        }
      }
    } 
  }
  
  // Methode zur berprfung ob weitereres Splitten sinnvoll ist;   
  private boolean weiterSplitten() {
    // nicht mehr splitten, falls Daten homogen sind oder in allen
    // Merkmalsausprgungen bereinstimmen
    if (datenliste.istHomogen() || datenliste.gleicheAuspraegungen()) {
      return false;
    }
    return true; 
  }
  
  // Rekursive Baumerzeugung mit mglicher Tiefenbegrenzung 
  public void erzeugeNaechsteEbene(int tiefe) {
    if ((tiefe != 0) && weiterSplitten()) {
      bestimmeSplitMerkmal();
      Datenliste datenLinks = datenliste.erzeugeTeilliste(splitMerkmal,splitSchwellenwert,true);
      Datenliste datenRechts = datenliste.erzeugeTeilliste(splitMerkmal,splitSchwellenwert,false);
      linkesKind = new Knoten(datenLinks,"<"+splitSchwellenwert);
      rechtesKind = new Knoten(datenRechts,">="+splitSchwellenwert);
      linkesKind.erzeugeNaechsteEbene(tiefe-1);
      rechtesKind.erzeugeNaechsteEbene(tiefe-1);
    }
  }
  
  // Klassifizieren eines Datenelements mithilfe des Baumes 
  public String klassifiziere(Datenelement d) {
    if (linkesKind == null) {
      return label;
    }
    else {
      if (d.getAuspraegung(splitMerkmal) < splitSchwellenwert) {
        return linkesKind.klassifiziere(d);
      }
      else {
        return rechtesKind.klassifiziere(d);
      }
    } 
  }  
  
  // Ausgabe des Baumes
  public String toString() {
    return this.print("",true);
  }
  
  // Ausgabe des Baumes
  private String info() {
    if (linkesKind == null) {
      return "["  +wert + "]  <" + label + ">";
    }
    else {
      return  "["  + wert + "|" + datenliste.getMerkmal(splitMerkmal) + "]";
    }
  }
  
  // Ausgabe des Baumes
  private String print(String pre, boolean links) {
    String ergebnis = "";
    if (pre.equals("")) {
      ergebnis = ergebnis + info() + System.getProperty("line.separator");         
    } 
    else {
      if (links) {
        ergebnis = ergebnis + pre + "---" + info() + System.getProperty("line.separator");       
      } 
      else {
        String new_pre = "";
        for (int i=0; i<pre.length()-1; i++) {
          new_pre = new_pre+pre.charAt(i);
        }
        new_pre = new_pre+"|";
        ergebnis = ergebnis + new_pre + "---" + info() + System.getProperty("line.separator");       
      } 
    } 
    if (rechtesKind != null) {
      if (pre.equals("")) {
        ergebnis = ergebnis + rechtesKind.print(pre+" |",true);
      } 
      else {
        ergebnis = ergebnis + rechtesKind.print(pre+"    |",true);
      } 
    }
    if (linkesKind != null) {
      if (pre.equals("")) {
        ergebnis = ergebnis + linkesKind.print(pre+"  ",false);
      } 
      else {
        ergebnis = ergebnis + linkesKind.print(pre+"     ",false);
      } 
    }
    return ergebnis;
  }
  
} 
    